
// Nombre del Namespace o package de todas las clases e instancias de clases creadras
// en esta aplicación web
var CHT = {};
CHT.jQuery = jQuery; // guardamos ref a instancia actual de jQuery

//
// Inicio de la aplicación web cht
// Contexto principal:
// Funciones públicas
// * CHT.insertaPresionPunto
// * CHT.actualizaPresionPunto

(function($cht) {

    var $ = $cht.jQuery;

    if (!window["console"]) {
        window.console = {};
    }
    if (!console.info) {
        console.info = function() {};
    }
    if (!console.debug) {
        console.debug = function() {};
    }
    if (!console.error) {
        console.error = function() {};
    }

    var  /* Variables de configuración de la base de datos de CartoDB */
        api_key = "f5a8acb218b2dc4262d216d4a5fc24c9006b0af6", /* no es necesaria para las consultas */
        user  = "lourdes",
        URL_TABLA_FRONTERA = "http://" + user + ".cartodb.com/tiles/cht_v",
        URL_SQL = "http://" + user + ".cartodb.com/api/v2/sql?";


    var map; // objeto map de leaflet

    var jqMap; // objeto jQuery con el elemento DIV de id = "map"

    var _currentPopup; // ref al último popup mostrado

    var fronteraLayer; // capa con

    var puntosLayerGroup;// capa con subcapas de las presiones de puntos
    var lineasLayerGroup;// capa con subcapas de las presiones de lineas
    var areasLayerGroup;// capa con subcapas de las presiones de areas
	
    var _formPresion; // será la unica instancia de formulario para presiones puntuales

    var _dataTable;

    var _googleLayer; // esta es la capa usada para editar geometrías.

    var _tiposOcultados = CHT.tiposOcultados = {};

    // punto de entrada a la aplicación: Suscripción al evento de "documento listo"
    $(document).ready(function() {
        // inicio tras la carga del documento html.
        // Todos los elementos HTML ya están cargados en la página
        console.info("Iniciando CHT");
        try {
            initCht();
        } catch (ex) {
            console.error("Error inciando CHT: " + ex.message);
            return;
        }
        console.info("CHT iniciado correctamente");
    });

    /**
     * iniciacilización del mapa y de las capas, Formulario, bocadillo y tabla
     */
    function initCht() {

        jqMap = $("#map");
        map = new L.Map(jqMap[0].id); // construimos un objeto L.Map pasando un ID de un DIV
        // ubicación entre Toledo y Ávila.
	    map.setView([40.2, -4.6], 7);

	    // Creación de capas base
	
	    // Capa base de google
	    _googleLayer = new L.Google('SATELLITE'); // ROADMAP, SATELLITE
	    map.addLayer(_googleLayer);
	
	    // capa base de cloudmade
	    var cloudmadeLayer = L.tileLayer(
			    'http://{s}.tile.cloudmade.com/{key}/22677/256/{z}/{x}/{y}.png', 
				{ key: 'BC9A493B41014CAABB98F0471D759707'}).addTo(map);


        /** Capa de tiles.
        * Configuración en CartoDB.
        * #cht_v{
            polygon-fill:#99CC00;
            line-color:#333333;
            line-width:3;
            polygon-opacity:0.2;
            line-opacity:.85;
            line-clip:false;
            polygon-clip:false;
        }
        */
        fronteraLayer = L.tileLayer(URL_TABLA_FRONTERA + '/{z}/{x}/{y}.png', {}).addTo(map);

        puntosLayerGroup = L.layerGroup().addTo(map);
	    lineasLayerGroup = L.layerGroup().addTo(map);
        areasLayerGroup = L.layerGroup().addTo(map);
	
	    var baseballIcon = L.icon({
			iconUrl: 'baseball-marker.png',
			iconSize: [32, 37],
			iconAnchor: [16, 37],
			popupAnchor: [0, -28]
		    });

        _formPresion = new $cht.FormularioPresion(
	        "formulario-puntos", map, formularioOkHandler, formularioCancelHandler);

        _dataTable = new $.Tabla("div-data", {
            columns: [
                {"name": "pmspcod", "caption": "COD"},
                {"name": "pmspnom", "caption": "Nombre"},
                {"name": "tipo", "caption": "Tipo"}
            ],
            onRender: function(rowIndex, colIndex, cellData, row) {
                if (colIndex==2) {
                    return $cht.getDescripcionGrupo(cellData);
                }
                return cellData;
            }
        });

	    $("#bto-nuevo").click(function() {
	        // alerna entre visible/invisible
            quitarOcultables();
		    _formPresion.nuevaPresion();
	    });

        $("#bto-tabla").click(function() {
            // alerna entre visible/invisible
            quitarOcultables();
            _dataTable.mostrar();

        });

	    // Para el bocadillo
	    var btoBorrar;
	
	    map.on("popupopen", function(e){
	        btoBorrar = $("#plantilla-popup .bto-borrar");
		    btoBorrar.click(function(e) {
		        alert("aaa");
		  });
		});

        // carga por primera vez de la presiones. Cada función usa una petición AJAX
        recargaPresionesPuntuales();
        recargaPresionesLineales();
	    recargaPresionesSuperficiales();
	
	    var baseMaps = {
       	    "Base Satélite Google": _googleLayer,
	        "Base Cloudmade": cloudmadeLayer
        };

        var overlayMaps = {
	        "Frontera de la CHT": fronteraLayer,
	        "Presiones lineales": lineasLayerGroup,
	        "Presiones superficiales": areasLayerGroup,
            "Presiones puntuales": puntosLayerGroup
        };
	
        L.control.layers(baseMaps, overlayMaps).addTo(map);

        // Establecemos un intervalo de 10 segundos para actualizar el mapa automaticamente
        window.setInterval(function() {
            console.info("actualización automatica");
            // actualización automatica cada 10 segundos
            recargaPresiones();
        }, 10000);

        $(".supertipos input").click(function() {
            var checked = this.checked;
            var parent = this.parentNode;
            var cssName = parent.className;
            CHT.tiposOcultados[cssName] = checked;
            recargaPresiones();
        });

    } // fin de inicialización

    function quitarOcultables() {
        $("#elementos-ocultos .ocultable").hide();
    }

    /**
     *  Método que será invocado desde el API de leafLet tras cargar cada una de las geometrías punto. lineas y areas).
     *  Las geometrías son cargadas tras las peticiones AJAX correspondientes.
     *  ver: recargarPunto, recargarLineas y recargarAreas
     */
    function onEachPresionFeature(feature, layer) {
        try {
            var loc = layer["_latlng"]; // caso de un punto
            if (typeof loc == "undefined") {
                var polig = feature.geometry.coordinates[0];
                var pto = polig[0]; // UPGRADE - calcular centroide del poligono
                if ($.isArray(pto[0])) {
                    pto = pto[0];
                }
                loc = new L.LatLng(pto[1], pto[0]);
            }
            layer.on("click", function(e) {
                var popupContentDiv = $cht.rellenaBocadillo(feature, editarFeature, borrarFeature);
                // popup común para todas las geometrías
                _currentPopup = L.popup()
                    .setLatLng(loc)
                    .setContent(popupContentDiv)
                    .openOn(map);
            });
        } catch (ex) {
            alert ("OJO: " + ex);
        }
    }

	/** Manejador del evento producido cuando se cierra el formulario con OK */
	function formularioOkHandler(evt) {
        // mostramos el "mapa normal" en el que se muestran todas las presoines
	    $("#map").show();

		var prop = evt.feature.properties;
		 
		var geomType = evt.feature.geometry.type;
		 
		if (geomType=="MultiPolygon") {
		    var shape = $cht.getSelectedShape();
            if (prop.cartodb_id===null) {
               insertaPresionSuperficial(prop, shape.getPath());
            } else {
		       actualizaPresionSuperficial(prop, shape.getPath());
            }

		} else if (geomType=="MultiLineString") {
		    var shape = $cht.getSelectedShape();
            if (prop.cartodb_id===null) {
                insertaPresionLineal(prop, shape.getPath());
            } else {
                actualizaPresionLineal(prop, shape.getPath());
            }
		   
		} else if (geomType=="Point") {
		    var punto = evt.feature.points[0];
		    if (prop.cartodb_id===null) {
                insertaPresionPunto(prop, punto);
            } else {
                actualizaPresionPunto(prop, punto);
            }
			
		} else {
		    alert("Error interno: Tipo de geometría desconocido: " + geomType);
		}
		 
	}
	
	
	/** Manejador del evento producido cuando se cierra el formulario con Cancel */
	function formularioCancelHandler(evt) {
	   $("#map").show();
	}
	
	/** Invocado cuando se pulsa 'editar' en el bocadillo */
    function editarFeature(feature) {
        
        // cerramos bocadillo actual
        _currentPopup._close();
        quitarOcultables();

        // ocultamos mapa
         $("#map").hide();
         
        // iniciamos edición en mapa nuevo 
        _formPresion.editar(feature, map);
		
    }

    function addToTable(geoJson) {
        if (_dataTable) {
            if (geoJson["features"]) {
                var i;
                for (i=0;i<geoJson.features.length;i++) {
                    addToTable(geoJson.features[i]);
                }
            } else if (geoJson["properties"]) {
                _dataTable.addRow(geoJson.properties);
            } else {
                console.error("dato no reconocido para la tabla: " + geoJson);
            }
        }
    }
	
	
	/** Invocado cuando se pulsa 'borrar' en el bocadillo */
    function borrarFeature(feature) {
        _currentPopup._close();
        var id = feature.properties.cartodb_id;

        var tableName =  geomToTableName(feature.geometry);
        var sql = "delete from " + tableName + " where cartodb_id = " + id;
        var url = URL_SQL + "q=" + sql + "&api_key=" + api_key;
        $.getJSON(url,
            function(result){
                recargaPresiones();
                console.info("OK. Borrado");
            });
    }

    /** Esta función llama a las tres funciones que se encargan de hacer las peticiones de presiones
     *  puntuales, lineales y superficiales.
     */
	function recargaPresiones() {
        _dataTable.clear();
	    recargaPresionesPuntuales();
		recargaPresionesLineales();
		recargaPresionesSuperficiales();
	}
		
    function recargaPresionesPuntuales() {
        recargarPuntos(puntosLayerGroup, onEachPresionFeature);
    }

	function recargaPresionesLineales() {
        recargarLineas(lineasLayerGroup, onEachPresionFeature);
    }
	
	function recargaPresionesSuperficiales() {
        recargarAreas(areasLayerGroup, onEachPresionFeature);
    }
	
    // Carga de presiones

    // Carga asíncrona de las presiones lineales
    // Se crea una nueva capa vacía, se añade al mapa y es devuelta por la función.
    // La capa será llenada de geometrías tras la respuesta ajax
    function recargarLineas(layerGroup, onEachPresionFeature) {

        selectGeometries("presiones_lineas",
    		function (geoData) {
    		    // Datos recibidos tras peticion ajax
    		    var layer = L.geoJson(geoData, {
    			    style: $cht.getEstiloLinea,
                    filter: filtrarPorGrupos,
                    onEachFeature: onEachPresionFeature
    		    });
                layerGroup.clearLayers();
                layerGroup.addLayer(layer);
                addToTable(geoData);
    	    }
    	);

    }


    // Carga asíncrona de las presiones lineales
    // Se crea una nueva capa vacía, se añade al mapa y es devuelta por la función.
    // La capa será llenada de geometrías tras la respuesta ajax
    function recargarAreas(layerGroup, onEachPresionFeature) {

        selectGeometries("presiones_poligonos",
    		function(geoData){
    		    // Datos recibidos tras peticion ajax
    		    var layer = L.geoJson(geoData, {
    			    style: $cht.getEstiloPoligono,
                    filter: filtrarPorGrupos,
                    onEachFeature: onEachPresionFeature
    		    });
    		    layerGroup.clearLayers();
                layerGroup.addLayer(layer);
                addToTable(geoData);
    		}
        );

    }

    // Carga asíncrona de las presiones lineales
    // Se crea una nueva capa vacía, se añade al mapa y es devuelta por la función.
    // La capa será llenada de geometrías tras la respuesta ajax
    function recargarPuntos(layerGroup, onEachPresionFeature) {

        selectGeometries("PRESIONES_PUNTOS",
    		function(geoData){
    		    // Datos recibidos tras peticion ajax
    		    var layer = L.geoJson(geoData, {
    			   style: function (feature) {
    				  return feature.properties && feature.properties.style;
    			   },
                   filter: filtrarPorGrupos,
    			   onEachFeature: onEachPresionFeature,
    			   pointToLayer: function (feature, latlng) {
                       var cssName = $cht.getEstiloCssPunto(feature);
                       var myIcon = L.divIcon({iconSize: [40,40],iconAnchor: [20,40], className: cssName});
                       return L.marker(latlng, {icon:myIcon});
    			   }
                });
                layerGroup.clearLayers();
                layerGroup.addLayer(layer);
                addToTable(geoData);
    	    }
    	);

    }

    /**
     * Comprueba si una presion debe ser mostrada en el mapa según el tipo de presión.
     * Usa CHT.tiposOcultados
     * @param feature
     * @param capa
     */
    function filtrarPorGrupos(feature, capa) {
        // TODO - comporbar si el grupo al que pertenece esta geometría es visible
        var tipo = feature.properties.tipo;
        var subTipo = ""+tipo;
        var superTipo = (""+tipo).substring(0,1) + "00";

        var esVisible = CHT.tiposOcultados["t-" + tipo];

        if (typeof(esVisible) == "undefined") {
            esVisible = CHT.tiposOcultados["t-" + superTipo];
        }
        return esVisible;
    }
    /**
     * Función común para peticion de datos sobre geometrías
     * @param tableName: nombre de la tabla:
     * @param onResultData: función que maneja el resultado
     */
    function selectGeometries(tableName, onResultData) {
        var url = 'https://lourdes.cartodb.com/api/v2/sql?format=GeoJSON&q=select * from ' + tableName + ' LIMIT 2000';
        try {
            $.getJSON(url, onResultData);
        } catch (err) {
            console.error("ERROR interno recuperando datos de: " + tableName, err);
        }

    }

    /**
     * Función que inserta unas presiones de tipo puntual en la base de datos
     * @param prop
     * @param point
     */
    function insertaPresionPunto(prop, point) {

        var geom =  toSql_Wgs84Point(point);
        insertaPresion(prop, "PRESIONES_PUNTOS", geom, "puntual");

    }

    /**
     * Función que inserta unas presiones de tipo puntual en la base de datos
     * @param prop
     * @param point
     */
    function insertaPresionLineal(prop, path) {

        var geom =  toSql_Wgs84Lineal(path);
        insertaPresion(prop, "PRESIONES_LINEAS", geom, "lineal");

    }

    /**
     * Función que inserta unas presiones de tipo puntual en la base de datos
     * @param prop
     * @param point
     */
    function insertaPresionSuperficial(prop, path) {

        var geom =  toSql_Wgs84MultiPolygon(path);
        insertaPresion(prop, "PRESIONES_POLIGONOS", geom, "superficial");

    }

    function insertaPresion(prop, nomTabla, geom, tipo) {

        var sql = "INSERT INTO " + nomTabla + " " +
            "(pmspcod, pmspnom, tipo, the_geom) VALUES (" +
            toSql_String(prop.pmspcod) + ", " +
            toSql_String(prop.pmspnom) + ", " +
            prop.tipo + ", " +
            geom +
            ")";

        enviarSql(sql,
            function(evt) {
                alert("Presión " + tipo + " insertada correctamente");
            });

    }

    /**
     * Función que debe ser usada para insertar una presion puntual en la base de datos
     * @param prop: propiedades de la presión (excepto la geometría)
     * @param point: geometría de tipo punto
     */
    function actualizaPresionPunto(prop, point) {

	    var geom =  toSql_Wgs84Point(point);
		actualizaPresion(prop, "PRESIONES_PUNTOS", geom, "puntual");
        
    }

	/**
     * Función que debe ser usada para insertar una presion puntual en la base de datos
     * @param prop: propiedades de la presión (excepto la geometría)
     * @param point: geometría de tipo punto
     */
    function actualizaPresionLineal(prop, points) {

	    var geom =  toSql_Wgs84Lineal(points);
		actualizaPresion(prop, "PRESIONES_LINEAS", geom, "puntual");
        
    }

	/**
     * Función que debe ser usada para insertar una presion puntual en la base de datos
     * @param prop: propiedades de la presión (excepto la geometría)
     * @param point: geometría de tipo punto
     */
    function actualizaPresionSuperficial(prop, path) {

	    var geom =  toSql_Wgs84MultiPolygon(path);
		actualizaPresion(prop, "PRESIONES_POLIGONOS", geom, "superficial");
        
    }

	
	  /**
     * Función que debe ser usada para insertar una presion puntual en la base de datos
     * @param prop: propiedades de la presión (excepto la geometría)
	 * @param nomTabla: nombre de la tabla
     * @param geom: geometría en formato Sql
	 * @param tipo: nombre del tipo de presion: puntual, lineal, superficial
     */
    function actualizaPresion(prop, nomTabla, geom, tipo) {

        var sql = "UPDATE " + nomTabla + " SET " +
            " pmspcod = " + toSql_String(prop.pmspcod) + ", " +
            " pmspnom = " + toSql_String(prop.pmspnom) + ", " +
            " tipo = " + prop.tipo + ", " +
            " the_geom = " +  geom +
            " WHERE cartodb_id = " + prop.cartodb_id;

        enviarSql(sql,
            function(evt) {
                alert("Presión " + tipo + " modificada correctamente");
            });

    }
	
	
    /**
     * Función que envia una sql non-query a CartoDB
     * @param sql
     */
    function enviarSql(sql, okHandler) {

        var url = URL_SQL + "q=" + sql + "&api_key=" + api_key;

        // TODO -  falta detectar cuándo el servidor devuelve un error

        $.getJSON(url,
            function(result){
                var n = result.total_rows;
                if (n==1) {
                    recargaPresiones();
                    okHandler.call(this);
                } else {
                    alert("Fallo interno en acceso a base de datos. ");
                }
            });
    }

    function toSql_String(text) {
        return "'" + text + "'";
    }

    function toSql_Wgs84Point(point) {
        var text = "ST_SetSRID(ST_Point(" + point[0] + ", " + point[1] + "),4326)";
        return text;
    }

    /** Función para crear un multipoligono de un solo poligono el cual solo tiene el anillo externo */
    function toSql_Wgs84Lineal(path) {

        var coords;
        var listaPuntos = "";

        for (var i = 0; i < path.length; i++) {
            coord = path.getAt(i);
            if (listaPuntos != "") listaPuntos += ","
            listaPuntos +=  coord.lng() + " " + coord.lat();
        }

        var mp = " ST_GeomFromText('MULTILINESTRING((" + listaPuntos + "))', 4326) ";
        return mp;

    }

	/** Función para crear un multipoligono de un solo poligono el cual solo tiene el anillo externo */
    function toSql_Wgs84MultiPolygon(path) {
   
      var coords;
	  var listaPuntos = "";
	
      for (var i = 0; i < path.length; i++) {
        coord = path.getAt(i);
	    if (listaPuntos != "") listaPuntos += ","
	    listaPuntos +=  coord.lng() + " " + coord.lat();
      }
	
	  var c0 = path.getAt(0);
      listaPuntos +=  "," + c0.lng() + " " + c0.lat();
      
      var mp = " ST_GeomFromText('MULTIPOLYGON(((" + listaPuntos + ")))', 4326) ";
      return mp;

    }

    /**
     * Obtiene el nombre de la tabla donde se almacena las presiones cuya geomatría
     * es pasa por parámetro
     * @param geom: geometría obtenido desde cartoDb
     */
    function geomToTableName(geom) {
        // TODO - declarar como etrcutrua json
        if (geom.type=="MultiLineString") return "PRESIONES_LINEAS";
        if (geom.type=="Point") return "PRESIONES_PUNTOS";
        if (geom.type=="MultiPolygon") return "PRESIONES_POLIGONOS";
        return "TABLA_DESCONOCIDA";
    }

}(CHT))

// Es requisito que exista el objeto CHT



