
/**
  *  Clouse para los editores de geometrias.
  *  Se usa un plugin de GoogleMaps  (libraries=drawing)
  */

(function($cht) {

   /*  Para guardar
       storePolygon(selectedShape.getPath(), selectedShape.cartodb_id);
	   */
	var $ = $cht.jQuery;
	
    $(document.body).ready(function() {
	    initEditores();
    });
	
    // TODO - comrpobar que está descargado el plugin "google.drawing"
    
    $cht.editarPunto = editarPunto;
    $cht.editarPoligono = editarPoligono;
	$cht.editarPolilinea = editarPolilinea;

    $cht.crearPunto = crearPunto;
    $cht.crearPolilinea = crearPolilinea;
    $cht.crearPoligono = crearPoligono;

    $cht.clearSelection = clearSelection;

    $cht.getSelectedShape = function() { return _selectedShape; };
	
	// variables internas
	var _gMap;
    var _selectedShape = null;
    var _tempMarkers = [];

    function initEditores() {
	
	  var lat=40.2, lng=-4.6, zoom= 7;
	     //Basic
      var cartodbMapOptions = {
        zoom: zoom,
        center: new google.maps.LatLng( lat, lng ),
        disableDefaultUI: true,
        mapTypeId: google.maps.MapTypeId.SATELLITE  //.ROADMAP
      }

      // Init the map
      _gMap = new google.maps.Map(document.getElementById("gmap"),cartodbMapOptions);

      // Define the map styles (optional)
      var mapStyle = [{
        stylers: [{ saturation: -65 }, { gamma: 1.52 }] }, {
        featureType: "administrative", stylers: [{ saturation: -95 }, { gamma: 2.26 }] }, {
        featureType: "water", elementType: "labels", stylers: [{ visibility: "off" }] }, {
        featureType: "administrative.locality", stylers: [{ visibility: 'off' }] }, {
        featureType: "road", stylers: [{ visibility: "simplified" }, { saturation: -99 }, { gamma: 2.22 }] }, {
        featureType: "poi", elementType: "labels", stylers: [{ visibility: "off" }] }, {
        featureType: "road.arterial", stylers: [{ visibility: 'off' }] }, {
        featureType: "road.local", elementType: "labels", stylers: [{ visibility: 'off' }] }, {
        featureType: "transit", stylers: [{ visibility: 'off' }] }, {
        featureType: "road", elementType: "labels", stylers: [{ visibility: 'off' }] }, {
        featureType: "poi", stylers: [{ saturation: -55 }]
      }];
	   

	}

    function initLocationAndZoom(map) {
        var center = map.getCenter();
        var zoom = map.getZoom();
        _gMap.setCenter(new google.maps.LatLng(center.lat, center.lng ));
        _gMap.setZoom(zoom);
    }

    /**
	 *
	 */
    function editarPunto(feature, map, updateMarker) {
        initLocationAndZoom(map);

        clearSelection();
		$("#gmap").show();
           
        var marker = createGoogleMarker(feature.geometry);
	    var id = feature.properties.cartodb_id;
		
        marker.cartodb_id = id;
        marker.setMap(_gMap);
        
        google.maps.event.addListener(marker, "dragend", function() {
            updateMarker(marker);
         });

        _selectedShape = marker;
       // var latlng = marker.getPoint();
       //cells.desc.innerHTML = "(" + Math.round(latlng.y * 100) / 100 + ", " +
 		
    }
   
    /**
	 *
	 */
    function editarPoligono(feature, map) {
        initLocationAndZoom(map);
		
        clearSelection();
		$("#gmap").show();
        var points = getExteriorRing(feature.geometry);
        var polig = createGooglePolygon(points);
	    var id = feature.properties.cartodb_id;
		
        polig.cartodb_id = id;
        polig.setMap(_gMap);
 
        setSelection(polig);
		
    }
	
     /**
	 *
	 */
    function editarPolilinea(feature, map) {
        initLocationAndZoom(map);
		
        clearSelection();
		$("#gmap").show();

        var points = getFirstPolyline(feature.geometry);
        var polilinea = createGooglePolilinea(points);
	    var id = feature.properties.cartodb_id;
		
        polilinea.cartodb_id = id;
        setSelection(polilinea);

        polilinea.setMap(_gMap);
		
    }


    /** Borra actual selección y marcadores temporales */
	function clearSelection() {
        if (!_selectedShape) {
           // no hay seleccionado
           return;
        }

        _selectedShape.setMap(null);
        _selectedShape = null;
    }

    function setSelection(shape) {
        clearSelection();

        _selectedShape = shape;
        shape.setEditable(true);
    }



    function getFirstPolyline(geomMultiPolyline) {

        var poli1, coords;
        poli1 = geomMultiPolyline.coordinates[0];
        if (!poli1 || poli1.length<2) {
            // no hay poligono ???
            return;
        }
        coords = poli1;
        var points = [];
        var j;
        for (j in coords) {
            points.push(new google.maps.LatLng(coords[j][1], coords[j][0]))
        }

        return points;
    }

    function getExteriorRing(geomPolygon) {

        var polis, coords;
        polis = geomPolygon.coordinates[0];
        if (!polis || polis.length<1) {
            // no hay poligono ???
            return;
        }
        coords = polis[0];
        var points = [];
        var j;
        for (j in coords) {
            points.push(new google.maps.LatLng(coords[j][1], coords[j][0]))
        }

        points.pop();
        return points;
    }
        
    /** Devuelve un google.map.Polygon a partir de un feature de tipo Multipolygon 
	  */
    function createGooglePolygon(points) {

        var options = { paths: points,
            strokeColor: '#AA2143',
            strokeOpacity: 1,
            strokeWeight: 2,
            fillColor: "#FFff00",
            fillOpacity: 0.7 };

       var poly = new google.maps.Polygon(options);
	   return poly;
 
    }
   
       /** Devuelve un google.map.Poliline ???? a partir de un feature de tipo Multipolygon 
	  */
    function createGooglePolilinea(points) {

        var options = { path: points,
            strokeColor: '#AA2143',
            strokeOpacity: 1,
            strokeWeight: 2
       };

       var poly = new google.maps.Polyline(options);
	   return poly;
 
    }
   
        /** Devuelve un google.map.Poliline ???? a partir de un feature de tipo Multipolygon 
	  */
    function createGoogleMarker(geometry) {
    
       var coords = geometry.coordinates;
	   if (!coords || coords.length<2) {
		    // no hay punto ???
			return;
       }

       var icon = new google.maps.MarkerImage("css/images/punto-editando.png", 
            null, null, new google.maps.Point(20, 40));
           
       var latlng = new google.maps.LatLng(coords[1], coords[0]);
       
       var marker = new google.maps.Marker( { position: latlng, icon: icon, draggable: true});
      
	   return marker;
 
    }
  
  function drawPolygon(id, poly) {
    // Construct the polygon
    // Note that we don't specify an array or arrays, but instead just
    // a simple array of LatLngs in the paths property
    var options = { paths: poly,
      strokeColor: '#AA2143',
      strokeOpacity: 1,
      strokeWeight: 2,
      fillColor: "#FFff00",
      fillOpacity: 0.7 };

      newPoly = new google.maps.Polygon(options);
      newPoly.cartodb_id = id;
      newPoly.setMap(map);
      google.maps.event.addListener(newPoly, 'click', function() {
        this.setEditable(true);
        setSelection(this);
      });

      polys.push(newPoly);
  }

    /** Invocado para comenzar el modo de selección de 'posición puntual' en el mapa
     *
     * @param map: Mapa original del que tomamos centro y zoom actuales
     * @param updateMarker: método (callback) que será invocado cada vez que seleccionemos o modifiquemos
     * la posición en el mapa.
     */
    function crearPunto(map, updateMarker) {
        initLocationAndZoom(map);

        clearSelection();
        $("#gmap").show();

        // suscripcion a evento clic en mapa
        var listener = google.maps.event.addListener(_gMap, "click", function(evt) {
            var latlng = evt.latLng;
            if (latlng) {
                google.maps.event.removeListener(listener);
                var geom = {coordinates: [ latlng.lng(), latlng.lat()]};
                var marker = createGoogleMarker(geom);
                marker.setMap(_gMap);
                _selectedShape = marker;
                updateMarker(marker);
                google.maps.event.addListener(marker, "dragend", function() {
                    updateMarker(marker);
                });
                google.maps.event.addListener(marker, "click", function() {
                    updateMarker(marker);
                });
            }
        });
    }
  
    function crearPoligono(map, updatePoligono) {

        initLocationAndZoom(map);
        var color = "#ffff55";

        var poly = new google.maps.Polyline({
            map: _gMap,
            path: [],
            strokeColor: "#FF0000",
            strokeOpacity: 1.0,
            strokeWeight: 2 });

        setSelection(poly);
        poly.cartob_id = null;

        var isClosed = false;

        var markers = [];

        var clic = google.maps.event.addListener(_gMap, 'click', function (clickEvent) {
            var markerIndex = poly.getPath().length;
            poly.getPath().push(clickEvent.latLng);
            isClosed = markerIndex > 1;
            if (isClosed) {

                google.maps.event.removeListener(clic);
                //quitamos del mapa la polilinea
                poly.setMap(null);
                var path = poly.getPath();
                //agregamospligono editable
                poly = new google.maps.Polygon({ map: _gMap, path: path, strokeColor: "#FF0000", strokeOpacity: 0.8, strokeWeight: 2, fillColor: "#FF0000", fillOpacity: 0.35 });
                poly.cartodb_id = null;

                setSelection(poly);

                google.maps.event.addListener(poly.getPath(), 'insert_at', function (clickEvent) {
                    updatePoligono(poly);
                });
                google.maps.event.addListener(poly.getPath(), 'remove_at', function (clickEvent) {
                    updatePoligono(poly);
                });

            }
            updatePoligono(poly);
        });

    }

    function crearPolilinea(map, updatePolilinea) {
        initLocationAndZoom(map);
        var color = "#ffff55";

        var poly = new google.maps.Polyline({
            map: _gMap,
            path: [],
            strokeColor: "#FF0000",
            strokeOpacity: 1.0,
            strokeWeight: 2 });

        poly.cartob_id = null;
        setSelection(poly);

        google.maps.event.addListener(_gMap, 'click', function (clickEvent) {
              poly.getPath().push(clickEvent.latLng);
              updatePolilinea(poly);
        });

        google.maps.event.addListener(poly.getPath(), 'insert_at', function (clickEvent) {
            updatePolilinea(poly);
        });
        google.maps.event.addListener(poly.getPath(), 'remove_at', function (clickEvent) {
            updatePolilinea(poly);
        });
    }

    function startDrawing(poly, color, onUpdate) {

        _selectedShape = poly;
        var options = {}; // ninguna
        poly.setMap(_gMap);
        //map.addOverlay(poly);
        //poly.enableDrawing(options);
        //poly.enableEditing({onEvent: "mouseover"});
        //poly.disableEditing({onEvent: "mouseout"});
        poly.setEditable(true);
        google.maps.event.addListener(poly, "endline", function() {

            //var cells = addFeatureEntry(name, color);
            google.maps.event.bind(poly, "lineupdated", "nueva presión", onUpdate);
            google.maps.event.addListener(poly, "click", function(latlng, index) {
                if (typeof index == "number") {
                    poly.deleteVertex(index);
                } else {
                    var newColor = "#66ff66";
                    poly.setStrokeStyle({color: newColor, weight: 4});
                }
            });
        });
    }




})(CHT)


    /*
     var clic = google.maps.event.addListener(_gMap, 'click', function (clickEvent) {
     if (isClosed) {
     markers[0].setMap(null);
     markers[1].setMap(null);
     google.maps.event.removeListener(clic);
     setSelection(poly);
     poly.cartob_id = null;
     var path = poly.getPath();
     alert("Cerrado");
     return;
     }
     var markerIndex = poly.getPath().length;
     var isFirstMarker = markerIndex === 0;
     var marker = new google.maps.Marker({ map: _gMap, position: clickEvent.latLng, draggable: true });
     markers[markerIndex] = marker;
     if (isFirstMarker) {
     google.maps.event.addListener(marker, 'click', function () {
     if (isClosed)
     return;
     var path = poly.getPath();
     poly.setMap(null);
     poly = new google.maps.Polygon({ map: _gMap, path: path, strokeColor: "#FF0000", strokeOpacity: 0.8, strokeWeight: 2, fillColor: "#FF0000", fillOpacity: 0.35 });
     isClosed = true;
     });
     }
     google.maps.event.addListener(marker, 'drag', function (dragEvent) {
     poly.getPath().setAt(markerIndex, dragEvent.latLng);
     });
     poly.getPath().push(clickEvent.latLng);
     });

        */