

//
// Closure para los formularios.
// Clases públicas:
// * Formulario para cualquier tipo de gemetría.
//
//
(function($cht) {

    var $ = $cht.jQuery;
    $cht.FormularioPresion = FormularioPresion;


    ///
    /// Clase formulario para cualquier tipo de geometría (punto, polilinea, poligono)
    ///
    function FormularioPresion(domId, map, okHandler, cancelHandler) {

        var _this = this; // Ref para uso en funciones (que no son métodos)

        // ocultamos formulario (si no o estaba)
        // De momento servirá solo para presiones 
        var jqMainDiv = $("#" + domId).hide();

        var jqMap = $(map._container);

		var _currentFeature;
		
        // Guardamos ref a elementos
        var _txtCoordenadas = jqMainDiv.find("#input-coordenadas");

        var _txtCod = jqMainDiv.find("#input-pmspcod");
        var _txtNombre = jqMainDiv.find("#input-pmspnom");
        var _txtGrupo = jqMainDiv.find("#input-tipo");

        CHT.rellenaComboGrupos(_txtGrupo[0]);

        var _btoFijarTipo = $("#bto-fijarTipo").click(function() {
            _btoFijarTipo.attr("disabled", true);
            var sGemoType = fijarTipoActual();
            activarCreacionGeometria(sGemoType);
        });

        var btoCancel = $("#bto-cancel");
        btoCancel.click(function() {
            $cht.clearSelection();
            jqMainDiv.hide();
			cancelHandler.call(this);
        });

        // Flag para saber si es nueva presión o modificación
        var _cartodb_id = null;

        var btoOk = $("#bto-aceptar");
        btoOk.click(function() {
            try {
                okClicadoHandler();
            } catch (ex) {
                alert("Error en la inserción \n\n: " + ex);
                return;
            }
            jqMainDiv.hide();
            // se cierra el formulario e invocamos al manejador de edición o nueva presión
			okHandler.call(this, { feature: _currentFeature });
        });

        /* Alterna entre visible / no visible el element DIV que contiene el formulario */
        this.toggle = function() {
            jqMainDiv.toggle();
        };

        /* Rellena el formulario con los datos del "feature" pasado como parámetro
         * El feature pasado como parámetro debe ser de geometría puntual  */
        this.editar = function(feature) {

            if (feature.type !== "Feature") {
                // Error interno.
                // mostrar logs
            }
            _currentFeature = feature;

            resetearTipoActual();

            var prop = feature.properties;
            _setCartodbId(prop.cartodb_id);
            _txtCod.val(prop.pmspcod);
            _txtNombre.val(prop.pmspnom);
            _txtGrupo.val(prop.tipo);

            // seleccionamos el radio button que corresponda a la geometría actual
            var geom = feature.geometry;
            if (geom.type == "Point") {
				 var sCoord = geom.coordinates[0] + " " + geom.coordinates[1];
                 _txtCoordenadas.val(sCoord);
				  jqMainDiv.find("#input-geom-punto").attr("checked", true);
			} else if (geom.type == "MultiLineString") {	
			      _txtCoordenadas.val("");
                  _txtCoordenadas.attr("disabled", "disabled");
			      jqMainDiv.find("#input-geom-linea").attr("checked", true);
            } else if (geom.type == "MultiPolygon") {
                 _txtCoordenadas.val("");
				 _txtCoordenadas.attr("disabled", "disabled");
				 jqMainDiv.find("#input-geom-poligono").attr("checked", true);
            } else {
			     _txtCoordenadas.hide();
			}
            
            jqMainDiv.show();
            
               
           var geomType = feature.geometry.type;
		   if (geomType=="MultiPolygon") {
		       CHT.editarPoligono(feature, map);
		   } else if (geomType=="MultiLineString") {
               CHT.editarPolilinea(feature, map);
  	       } else if (geomType=="Point") {
               CHT.editarPunto(feature, map, function(marker) {
                   var loc = marker.position;
                   var sCoord = loc.lng() + " " + loc.lat();
                  _txtCoordenadas.val(sCoord);
               });
		   }
            
            
        };

        /**
         * Función que establece cuál es el id de la presión actualmente en edición.
         * Si el id es null significa que el formulario se está usando para insertar una
         * nueva presión
         * @param num: un número o null
         */
        function _setCartodbId(num) {
            _cartodb_id = num;
            if (num===null) {
                _txtCod.prop("disabled", false);
            } else {
                _txtCod.prop("disabled", true);
            }
        }

        /** Método Usado para saber si el formulario mostrado actualmente representa
         * una nueva presión o una presión existente
         */
        this.esNuevo = function() {
            return _cartodb_id === null;
        }

        /**
         *  Método que muestra el formulario para introducir datos de una nueva presión
         */
        this.nuevaPresion = function() {
            resetearCampos();
            resetearTipoActual();
            _setCartodbId(null);
            jqMainDiv.show();
        };

        /* Manejador del evento de clic en el botón OK */
        function okClicadoHandler() {
		
		    var feature = _currentFeature;
			
            var prop = {};
			prop.type = feature.type;
            prop.cartodb_id = _cartodb_id;
            prop.pmspcod = _txtCod.val();
            prop.pmspnom = _txtNombre.val();
            prop.tipo = _txtGrupo.val();
            validarDatos(prop);

            var ns = (_txtCoordenadas.val() + " A B").split(' ');
            var punto = [ ns[0], ns[1] ];
            validarPunto(prop);

			feature.properties = prop;
			// depende de fetaure type
			feature.points = [punto];

        }

        function validarDatos(prop) {
            // validando código
            var LIM_DIGITOS = 12;
            if (prop.pmspcod === "") {
                throw "Campo código debe tener algún texto";
            }
            if (prop.pmspcod.length>LIM_DIGITOS) {
                throw "Campo código no debe exceder de " + LIM_DIGITOS+ " caracteres";
            }
            if (!isNumeric(prop.pmspcod)) {
                throw "El código debe ser un número (positivo y de hasta " + LIM_DIGITOS + " digitos";
            }

            // TODO -  comprobar decimales y mayor que cero

            // validando nombre
            if (prop.pmspnom === "") {
                throw "Campo nombre debe tener algún texto";
            }
        }

        function isNumeric(text) {
            return (String(text).search(/^\d+$/) != -1);
        }

        function validarPunto(p) {

        }

        function resetearCampos() {
            _txtCoordenadas.val("");
            _txtCod.val("");
            _txtNombre.val("");
            _txtGrupo.val("");
        }

        var _CHECK_NAMES = ["linea", "punto" , "poligono" ];


        function resetearTipoActual() {
            var i=0;
            for (;i<_CHECK_NAMES.length; i++) {
                var ch = jqMainDiv.find("#input-geom-" + _CHECK_NAMES[i]);
                ch.attr("disabled", false);
                var sColor = "#114411";
                ch.next().css("color", sColor);
            }
            _btoFijarTipo.attr("disabled", false);
        }

        /** Se ha confirmado el tipo de geometría.
         *  Entonces es mostrado el mapa para insertar nueva geometría
         */
        function fijarTipoActual() {

             var i=0;
             var geomType = "";
             for (;i<_CHECK_NAMES.length; i++) {
                var ch = jqMainDiv.find("#input-geom-" + _CHECK_NAMES[i]);
                var dis = ch[0].checked;
		        ch.attr("disabled", !dis);
                var sColor = dis ? "#114411" : "#997777";
                ch.next().css("color", sColor);
                if (dis) geomType = _CHECK_NAMES[i];
             }
            return geomType;

        }

        function activarCreacionGeometria(geomType) {
            $("#map").hide();  // usar una referencia más segura en lugar de @map

            _currentFeature = {};
            _currentFeature.cartodb_id = null;
            _currentFeature.geometry = {};

            _txtCoordenadas.attr("disabled", true);

            if (geomType=="poligono") {
                _currentFeature.geometry.type = "MultiPolygon";
                CHT.crearPoligono(map, function(polig) {
                    var n = polig.getPath().length;
                    var sCoord = n + " puntos";
                    if (n<3) {
                        sCoord += ". Faltan al menos " + (3-n);
                    }
                    _txtCoordenadas.val(sCoord);
                });
            } else if (geomType=="linea") {
                _currentFeature.geometry.type = "MultiLineString";
                _txtCoordenadas.val("0 puntos. Faltan al menos 2.");
                CHT.crearPolilinea(map, function(poli) {
                    var n = poli.getPath().length;
                    var sCoord = n + " puntos";
                    if (n<2) {
                       sCoord += ". Faltan al menos " + (2-n);
                    }
                    _txtCoordenadas.val(sCoord);
                });
            } else if (geomType=="punto") {
                _currentFeature.geometry.type = "Point";
                CHT.crearPunto(map, function(marker) {
                    var loc = marker.position;
                    var sCoord = loc.lng() + " " + loc.lat();
                    _txtCoordenadas.val(sCoord);
                });
            }

        }

    }


})(CHT)