
//
// Closure para widget Tabla.
// Muestra una tabla de datos dentro de un elemento HTML DIV
//

(function($) {


    $.Tabla = Tabla;


    function Tabla(nodeId, options) {

        var _this = this;
        var _domId = document.getElementById(nodeId);

        var _title = options.title || "Tabla de datos";

        var _titleDiv = $('<p class="titulo">Listado de presiones</p>')[0];
        var _domTable = $('<table id="t-data" cellpadding="1px" cellspacing="1px"></table>')[0];

        _domId.appendChild(_titleDiv);
        _domId.appendChild(_domTable);

        $(_titleDiv).click(function(evt) {
            _this.ocultar();
        });
        this.clear = function() {
            $(_domTable).find("tr.row").remove();
        };
        this.mostrar = function() {
            $(_domId).show();
        }
        this.ocultar = function() {
            $(_domId).hide();
        }
        this.addRow = function(data) {
            addRow(data, _domTable, options)
        };
        this.fillTable = function(rows, clearBeforeFill) {
            if (!!clearBeforeFill) {
                this.clear();
            }
            fillTable(rows, _domTable, options);
        }
        initTable(options.columns)

        // private funcinos

        function initTable(columns) {
            var sCols = "";
            var i;
            for (i=0; i<columns.length;i++) {
                sCols += "<th>" + columns[i].caption + "</th>";
            }
            // add an extra columna for index or row
            _domTable.innerHTML = "<tr class='head'><th>&nbsp;</th>" + sCols + "</tr>";
        }

        function fillTable(rows, domTable, options) {

            var text = "";
            var columns = options.columns || [];
            var i;
            for (i=0; i<rows.length; i++) {
                var row = rows[i];
                text += "<tr class='row' id='r-" + i + "' >";
                text += "<td class='i'>" + (i+1) + "</td>";
                var c;
                for (c=0; c<columns.length; c++) {
                    var colName = columns[c].name;
                    var sValue = row[colName];
                    if (options.onRender) {
                        sValue = options.onRender.call(_this, i+1, c, sValue, row);
                    }
                    text += "<td>" + row[colName] + "</td>";
                }
                text +="</tr>\n";
            }
            var node = domTable.childNodes[0];
            if (node.tagName!="tbody") {
                node = domTable;
            }
            node.innerHTML += text;
            initTableListeners(domTable);
        }

        function addRow(rowData, domTable, options) {
            var text = "";
            var rowCount = domTable.rows.length;
            var row = domTable.insertRow(rowCount);
            var columns = options.columns || [];
            row.className = "row";

            var colIndex = row.insertCell(0);
            colIndex.className = "bto";
            colIndex.innerHTML = "" + rowCount;
            colIndex.title = "Centrar en mapa";
            colIndex.addEventListener('click', function(evt) {
                var elem = evt.currentTarget.parentNode;
                var sn = elem.id.substring(2);
                var n = parseInt(sn);
                alert("Pendiente centrar presión en mapa");
            }, false);

            var c;
            for (c=0; c<columns.length; c++) {
                var colName = columns[c].name;
                var celda = row.insertCell(c+1)
                var sValue = rowData[colName];
                if (options.onRender) {
                    sValue = options.onRender.call(_this, rowCount, c, sValue, row);
                }
                celda.innerHTML = sValue;
            }

        }

        function initTableListeners(domTable) {
            var filas = domTable.rows;
            for (var h=1; h<filas.length; h++) {
                var fila = filas[h];
                var colu0 = fila.cells[0];
                colu0.addEventListener('click', function(evt) {
                    var elem = evt.currentTarget.parentNode;
                    var sn = elem.id.substring(2);
                    var n = parseInt(sn);
                    alert("pulsado");
                }, false);
            }
        }

        function selectRow(index) {
            var domTable = document.getElementById("t-data");
            var filas = domTable.rows;
            var fondoNormal = "";
            var fondoElegido = "#e0e0ff";
            var idElegido = "r-" + index;
            for (var h=1; h<filas.length; h++) {
                var fila = filas[h];
                var color = fondoNormal;
                if (fila.id == idElegido) {
                    color = fondoElegido;
                }
                fila.style.backgroundColor = color;
            }
        }

    }



})(jQuery)



